{% load i18n %}

<!-- Nav tabs -->
<ul class="nav nav-tabs" role="tablist">
    <li role="presentation" class="active">
        <a href="#compliance" aria-controls="compliance" role="tab" data-toggle="tab">
            {% translate 'User Compliance' %}
        </a>
    </li>

    <li role="presentation">
        <a href="#skill_sets" aria-controls="skill_sets" role="tab" data-toggle="tab">
            {% translate 'Skill Sets' %}
        </a>
    </li>
 </ul>
