/* Global styles shared by all apps */

/* attempting to fix AA's wrong usage of bootstrap classes
----------------------------------------------------------------------------------------------------------------------*/
@media all {
    .aa-memberaudit {
        margin-left: 15px;
    }
}

@media all and (min-width: 768px) {
    .aa-memberaudit {
        margin-left: 0;
    }
}

@media all {
    /* Bootstrap Tabs */
    .template-light-mode .nav-tabs > li.active > a {
        background-color: rgb(236, 240, 241) !important;
        color: rgb(0, 0, 0) !important;
    }

    .template-dark-mode .nav-tabs > li.active > a {
        background-color: rgb(70, 69, 69) !important;
        color: rgb(255, 255, 255) !important;
    }

    .panel-tabs {
        border-top: none;
        border-top-left-radius: 0%;
        border-top-right-radius: 0%;
    }

    /* Datatables row group headers */
    .template-light-mode .table > tbody > .table-group > td {
        background-color: rgb(105, 105, 105);
        color: rgb(255, 255, 255);
        font-weight: bold;
    }

    .template-dark-mode .table > tbody > .table-group > td {
        background-color: rgb(105, 105, 105);
        color: rgb(255, 255, 255);
        font-weight: bold;
    }

    /* other */
    .nowrap {
        white-space: nowrap;
    }
}

@media all and (min-width: 768px) {
    /* Bootstrap modals */
    .modal-xl {
        width: 90%;
        max-width: 1200px;
    }
}
