{% load i18n %}

<!-- user compliance -->
<div role="tabpanel" class="tab-pane active" id="compliance">
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="tab_compliance">
            <thead>
                <tr>
                    <th>{% translate 'Main' %}</th>
                    <th>{% translate 'Organization' %}</th>
                    <th>{% translate 'Registered?' %}</th>
                    <th>{% translate 'Compliant?' %}</th>
                    <th>{% translate 'Total Chars' %}</th>
                </tr>
            </thead>

            <tbody></tbody>
        </table>
    </div>
    <p class="text-muted">Color code: <span class="text-success">fully compliant</span> | <span class="text-warning">partly compliant</span> | <span class="text-danger">not compliant</span></p>
</div>
