# Generated by Django 3.1.4 on 2020-12-24 12:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0004_effect_longer_name"),
        ("memberaudit", "0002_add_share_permission"),
    ]

    operations = [
        migrations.CreateModel(
            name="EveShipType",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("eveuniverse.evetype",),
        ),
        migrations.CreateModel(
            name="EveSkillType",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("eveuniverse.evetype",),
        ),
        migrations.AddField(
            model_name="characterupdatestatus",
            name="parent_task_id",
            field=models.CharField(
                db_index=True,
                default="",
                help_text="ID of character_update task that started this update",
                max_length=36,
            ),
        ),
        migrations.AddField(
            model_name="characterupdatestatus",
            name="root_task_id",
            field=models.CharField(
                db_index=True,
                default="",
                help_text="ID of update_all_characters task that started this update",
                max_length=36,
            ),
        ),
        migrations.AlterField(
            model_name="characterupdatestatus",
            name="finished_at",
            field=models.DateTimeField(db_index=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="characterupdatestatus",
            name="started_at",
            field=models.DateTimeField(db_index=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name="skillset",
            name="ship_type",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Ship type is used for visual presentation only. All skill requirements must be explicitly defined.",
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="memberaudit.eveshiptype",
            ),
        ),
        migrations.AlterField(
            model_name="skillsetskill",
            name="eve_type",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="memberaudit.eveskilltype",
                verbose_name="skill",
            ),
        ),
    ]
